package RPM::Source::Transform;

use strict;
use warnings;
use Cwd;
use Carp;
use Getopt::Long;
use File::Basename;
use RPM::Source::Editor;

our $VERSION=0.011;

our @SUPPORT_CLASSES=qw/
RPM::Source::Transformation::DefaultPlayer
/;
our @RESOURCE_CLASSES=qw/
RPM::Source::Transformation::DebugResource
/;
our @FACTORY_CLASSES=qw//;

our $PLAYER='RPM::Source::Transformation::DefaultPlayer';
our $ParentProvider;
our $WRITER;

use RPM::Source::Transformation::Collection;
use RPM::Source::Transformation::DebugResource;
use RPM::Source::Transformation::DefaultPlayer;

our @INSRC;
my $help;

# TODO: make them $opt{}'ed and documented in help
our ($opt_sourcedir,$opt_tmpdir,$opt_safe_mode);
#### used, like in srpmconvert-fc or jppconvert
our $cleanup=1;
our $verbose=1;

sub new {
    my $class = shift;
    my $self = {
	'-specargs' => [],
	@_
    };
    bless $self, $class;
    my $is_embedded=$self->{-embedded};
    unless ($is_embedded) {
	$self->get_options();
    }
    $self->process_options();
    $self->verify_options();
    $self->{-resourceobj}=RPM::Source::Transformation::Resource->new();
    $self->_init_transformations();
    unless ($is_embedded) {
	$self->process_args();
	$self->verify_args();
    }
    return $self;
}

our @EXT_OPTS_USAGE;

sub usage {
    my ($self)=@_;
    print "General options:
	-h, --help
	-v, --verbose (can be specified multiple times)
	-q, --quiet

Input options:
	[--rpm] /path/to/src.rpm
	[--spec] /path/to/spec
";
    &longopt_usage();
    foreach my $usage (@EXT_OPTS_USAGE) {
	print "\n";
	&$usage();
    }
}

sub longopt_usage {
    print "
Processing options:
  SRPM unpacking options:
	--safe-mode restrain of doing unsafe operations such as spawning rpm
	--tmpdir /path/ - alternative tmpdir if default tmpdir is not big enough.
	--no-cleanup (do not remove tmp files; useful for debugging)
";
    map {$_->longopt_usage} @SUPPORT_CLASSES, @RESOURCE_CLASSES, @FACTORY_CLASSES;
}

our @LONGOPT=(
    'cleanup!'	=> \$cleanup,
    'rpm=s'	=> sub { push @INSRC, [ SOURCERPM => $_[1]]},
    'safe-mode'	=> \$opt_safe_mode,
    'sourcedir=s' => sub {$opt_sourcedir=$_[1]},
    'spec=s'	=> sub { push @INSRC, defined $opt_sourcedir ? [SPECFILE => $_[1], SOURCEDIR=> $opt_sourcedir] : [ SPECFILE => $_[1]]},
    'tmpdir=s'	=> \$opt_tmpdir,
);

our @PROTECTED_OPTIONS=(
    'help'  => \$help,
    'v|verbose+'  => \$verbose,
    'q|quiet'   => sub { $verbose = 0 },
);
our @EXT_OPTS;

sub get_longopt {
    my ($self)=@_;
#    print STDERR "DEBUG:SUPPORT_CLASSES:\n\t",join("\n\t", @RPM::Source::Transform::SUPPORT_CLASSES),"\n";
#    print STDERR "DEBUG:ParentProvider=$RPM::Source::Transform::ParentProvider\n";
    my @ret_longopt=@LONGOPT;
    push @ret_longopt, map {$_->get_longopt} @SUPPORT_CLASSES, @RESOURCE_CLASSES, @FACTORY_CLASSES;
    push @ret_longopt, @EXT_OPTS;
    return @ret_longopt;
}

sub verify_options {}

# can be used both as class and object method (for Embedded usage)
# used as class method in octave-package-builder
sub get_options {
    my ($self)=@_;
    my $result = GetOptions ($self->get_longopt,@PROTECTED_OPTIONS);
    if (not $result) {
	print "run $0 --help for details.\n";
	exit 1;
    } elsif ($help) {
	$self->usage();
	exit 0;
    }
}

sub process_options {
    my ($self)=@_;
    map {$_->process_options} @SUPPORT_CLASSES, @RESOURCE_CLASSES, @FACTORY_CLASSES;
}

sub process_args {
    my ($self)=@_;
    map {push @INSRC, &__process_srcarg("infile", $_, $opt_sourcedir)} @ARGV;
}

sub verify_args {
    my ($self)=@_;
    if (not @INSRC) {
	print STDERR "ERROR: please, specify [ --rpm | --spec ] /path/to/file argument.\n";
	$self->usage();
	exit (4);
    }
}

sub __process_srcarg {
    my $insrc=$_[1];
    my $sourcedir=$_[2];
    if ($insrc=~/\.src\.rpm$/i) {
	return [ SOURCERPM => $insrc];
    } elsif ($insrc=~/\.spec$/i) {
	return defined $sourcedir ? [SPECFILE => $insrc, SOURCEDIR=> $sourcedir] : [ SPECFILE => $insrc];
    } elsif ($insrc eq 'none') {
	return [ SPECFILE => undef];
    } else {
	die "$_[0]: $insrc does not look like src.rpm or spec.\n";
    } 
}

sub mk_opt {
#    my ($self)=@_;
    ### TODO: use some fixed keys hash here
    my %opt=(
#	- => $opt_,
	);
    map {$_->register_messages(\%opt)} @SUPPORT_CLASSES, @RESOURCE_CLASSES, @FACTORY_CLASSES;
    return \%opt;
}

sub _init_transformations {
    my ($self)=@_;
    my $resourceobj=$self->{-resourceobj};
    map {$_->init_factory($resourceobj)} @FACTORY_CLASSES;
}

sub transform {
    my ($self)=@_;
    while (@INSRC) {
	my $opt_transform=$self->mk_opt();
	# debug opt
	# use Data::Dumper;
	# print Dumper($opt);
	my $spec=$self->get_spec(shift @INSRC);
	my $parent;
	$parent=$self->get_parent($ParentProvider->get()) if $ParentProvider;
	$self->transform_once($spec,$parent,$opt_transform);
    };
}

sub transform_once {
    my ($self,$spec,$parent,$opt)=@_;

    my $resourceobj=$self->{-resourceobj};
    map {$_->prepare_resource($spec,$parent,$opt,$resourceobj)} @RESOURCE_CLASSES;

    my $actions=RPM::Source::Transformation::Collection->new();
    $actions->add(map {$_->transformations($opt,$resourceobj)} @FACTORY_CLASSES);

    if ($PLAYER->apply_collection($spec,$parent,$opt,$resourceobj,$actions) == RPM::Source::Transformation::Component::TRANSFORMATION_FAILURE) {
	goto CLEANUP;
    }

    my $write_rpm_ok;
    if ($WRITER) {
	if ($opt_safe_mode and exists $opt->{-write_hasher_tar} and not $opt->{-write_hasher_tar}) {
	    print STDERR "WARNING: writing hasher tar due to the safe mode.\n";
	    $opt->{-write_hasher_tar}=1;
	}
	$write_rpm_ok=$WRITER->write($spec,$parent,$opt,$resourceobj);
    }
  CLEANUP:
    $parent->cleanup() if $parent;
    $spec->cleanup() if $write_rpm_ok || $cleanup;
    
}

sub get_spec {
    my ($self,$inref)=@_;
    my $spec;
    return $spec unless $inref;
    my @tmpdir_opts;
    if (not $cleanup and $inref->[0] eq 'SOURCERPM') {
    # all those tmp_dirname hacks are just to have the RPM::Source::Editor's guts at hand on crash.
	my $tmp_dirname=basename($inref->[1]);
	$tmp_dirname=~s!.src.rpm!!;
	$tmp_dirname=cwd()."/$tmp_dirname.$$";
	push @tmpdir_opts, 	TMPDIR => $tmp_dirname;
    }
    $spec=RPM::Source::Editor->new(
	VERBOSE=> $verbose,
	CLEANUP=> $cleanup,
	-tmpdir => $opt_tmpdir,
	-use_rpmbuild_be => $opt_safe_mode ? 0: 1,
	-use_shell_eval => $opt_safe_mode ? 0: 1,
	@tmpdir_opts,
	@{$self->{'-specargs'}},
	@$inref,
	);
    die "FATAL: failed to load the file ".$inref->[1].".\n" unless $spec;
    return $spec;
}

sub get_parent {
    my ($self, $parentref)=@_;
    my $parent;
    if ($parentref and defined $parentref->[1]) {
	my $parentfile=$parentref->[1];
	die "FATAL: file does not exist: $parentfile.\n" unless -e $parentfile;
	$parent=RPM::Source::Editor->new(
	    VERBOSE=> $verbose,
	    -tmpdir => $opt_tmpdir,
	    -use_rpmbuild_be => $opt_safe_mode ? 0: 1,
	    @$parentref,
	    );
	die "FATAL: failed to load the parent $parentfile.\n" unless $parent;
    }
    return $parent;
}

1;
